@@ -105,6 +105,7 @@ class DeformableAttention2D(nn.Module):
     ):
         super().__init__()
         offset_scale = default(offset_scale, downsample_factor)
+        assert offset_kernel_size >= downsample_factor, 'offset kernel size must be greater than or equal to the downsample factor'
         assert divisible_by(offset_kernel_size - downsample_factor, 2)
 
         offset_groups = default(offset_groups, heads)
@@ -130,8 +131,8 @@ class DeformableAttention2D(nn.Module):
         self.rel_pos_bias = CPB(dim // 4, offset_groups = offset_groups, heads = heads, depth = 2)
 
         self.dropout = nn.Dropout(dropout)
-        self.to_q = nn.Conv2d(dim, inner_dim, 1, bias = False)
-        self.to_kv = nn.Conv2d(dim, inner_dim * 2, 1, bias = False)
+        self.to_q = nn.Conv2d(dim, inner_dim, 1, groups = offset_groups, bias = False)
+        self.to_kv = nn.Conv2d(dim, inner_dim * 2, 1, groups = offset_groups, bias = False)
         self.to_out = nn.Conv2d(inner_dim, dim, 1)
 
     def forward(self, x, return_vgrid = False):
@@ -152,8 +153,10 @@ class DeformableAttention2D(nn.Module):
 
         # calculate offsets - offset MLP shared across all groups
 
-        grouped_feats = rearrange(q, 'b (g d) ... -> (b g) d ...', g = self.offset_groups)
-        offsets = self.to_offsets(grouped_feats)
+        group = lambda t: rearrange(t, 'b (g d) ... -> (b g) d ...', g = self.offset_groups)
+
+        grouped_queries = group(q)
+        offsets = self.to_offsets(grouped_queries)
 
         # calculate grid + offsets
 
@@ -163,7 +166,7 @@ class DeformableAttention2D(nn.Module):
         vgrid_scaled = normalize_grid(vgrid)
 
         kv_feats = F.grid_sample(
-            grouped_feats,
+            group(x),
             vgrid_scaled,
         mode = 'bilinear', padding_mode = 'zeros', align_corners = False)
 
